#!/bin/sh

# a string that is longer than 63 characters but otherwise is a valid label
longer63="A12345678901234567890123456789012345678901234567890123456789012345"

# a string that is a long valid label
long="A123456789012345678901234567890123456789012345678901234567890"

# $1: test ID
# $2: description
# $3: optional extra argument for ./test
runtest()
{
	echo "$2..."
	./test "$1" "$3" 2>$1.log >$1.out
	if test -f "$1.OUT"
	then
		diff $1.out $1.OUT && echo "   ...OK" || echo " **** FAILED ****"
	else
		diff $1.out FAIL && echo "   ...OK" || echo " **** FAILED ****"
	fi
}

echo""
echo "--- Valid requests:"
# test emitting requests
echo "Request: host->ip..."
./test req.0 igor2.repo.hu > req-valid.0.new
diff req-valid.0.new req-valid.0 && echo "   ...OK" || echo " **** FAILED ****"

echo "Request: ip->host..."
./test req.1 152.66.115.1 > req-valid.1.new
diff req-valid.1.new req-valid.1 && echo "   ...OK" || echo " **** FAILED ****"

echo""
echo "--- Valid answers:"
# test valid answers
runtest ans-valid.0 "valid host->ip"
runtest ans-valid.1 "valid ip->host"

echo""
echo "--- Error conditions:"
# test broken answers and invalid requests
runtest ans-err-short.0  "Answer: short"
runtest ans-err-2short.0 "Answer: shorter than header"
runtest ans-err-noans.0  "Answer: not an answer"
runtest ans-err-norec.0  "Answer: non-recursive answer"
runtest ans-err-qd0.0    "Answer: request count 0"
runtest ans-err-an0.0    "Answer: answer count 0"
runtest ans-err-dlen3.0  "Answer: answer dlen 3 for IPv4"
runtest req-err-63.0     "Request: label longer than 63 characters"   "$longer63.com"
runtest req-err-255.0    "Request: name longer than 255 characters"   "$long.$long.$long.$long.$long.com"
runtest req-err-ipn.1    "Request: too long ip number"   "123.123.123.1234"


